@@ -2,10 +2,10 @@
 # Before `make install' is performed this script should be runnable with
 # `make test'. After `make install' it should work as `perl test.pl'
 
-# $Id: test.pl,v 1.15 2002/04/08 04:39:06 jrennie Exp $
+# $Id: test.pl,v 1.17 2002/06/11 13:08:35 jrennie Exp $
 
 my $i = 1;
-BEGIN { $| = 1; print "v1.6: 1..20\nv1.7: 1..22\n"; }
+BEGIN { $| = 1; print "v1.6: 1..24\nv1.7: 1..26\n"; }
 END { print "not ok 1\n" unless $loaded; }
 use WordNet::QueryData;
 $loaded = 1;
@@ -18,7 +18,7 @@ print "ok ", $i++, "\n";
 print "Loading index files.  This may take a while...\n";
 # Uses $WNHOME environment variable
 my $wn = WordNet::QueryData->new;
-#my $wn = WordNet::QueryData->new("/usr/local/wordnet-1.7/dict", 1);
+#my $wn = WordNet::QueryData->new("/home/ai2/jrennie/usr/share/wordnet-1.6/dict");
 my $ver = $wn->version();
 print "Found WordNet database version $ver\n";
 
@@ -42,20 +42,30 @@ scalar $wn->querySense ("cat#n#1", "hypo") == 2
     ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
 scalar $wn->querySense ("cat#noun#7", "syns") == 5
     ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
-scalar $wn->valid_forms ("lay down#v") == 2
+
+($wn->querySense ("play down"))[0] eq "play_down#v"
     ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
-scalar $wn->valid_forms ("checked#v") == 1
+($wn->querySense ("play down#v"))[0] eq "play_down#v#1"
     ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
 
-($wn->query ("cat#n#1", "glos") eq "feline mammal usually having thick soft fur and being unable to roar; domestic cats; wildcats  ") ? print
-"ok ", $i++, "\n" : print "not ok ", $i++, "\n";
-
-scalar $wn->query ("child#n#1", "syns") == 12
+($wn->queryWord ("play down#v", "ants"))[0] eq "play_up#v"
+    ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
+($wn->querySense ("lay down#v#1", "syns"))[0] eq "lay_down#v#1"
+    ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
+scalar $wn->validForms ("lay down#v") == 1
+    ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
+scalar $wn->validForms ("checked#v") == 1
     ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
 
-(([$wn->validForms ("lay down#2")]->[0]) eq "lay down"
-    and ([$wn->validForms ("ghostliest#3")]->[0]) eq "ghostly"
-    and ([$wn->validForms ("farther#4")]->[1]) eq "far")
+my ($foo) = $wn->querySense ("cat#n#1", "glos");
+($foo eq "feline mammal usually having thick soft fur and being unable to roar; domestic cats; wildcats  ") ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
+
+scalar $wn->querySense ("child#n#1", "syns") == 12
+    ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
+
+(([$wn->validForms ("lay down#2")]->[0]) eq "lay_down#v"
+    and ([$wn->validForms ("ghostliest#3")]->[0]) eq "ghostly#a"
+    and ([$wn->validForms ("farther#4")]->[0]) eq "far#r")
     ? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
 
 ($wn->querySense("authority#n#4", "attr"))[0] eq "certain#a#2"
@@ -81,6 +91,6 @@ if ($ver eq "1.6") {
 	? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
     scalar $wn->offset("0#n#1") == 11356314
 	? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
-    scalar $wn->forms("axes#1") == 4
+    scalar $wn->forms("axes#1") == 2
 	? print "ok ", $i++, "\n" : print "not ok ", $i++, "\n";
 }
